#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_security.interfaces import IPrincipalInfo, ADMIN_USER_ID
from pyams_security.interfaces.profile import PUBLIC_PROFILE_KEY, IPublicProfile
from pyams_utils.interfaces import PUBLIC_PERMISSION
from pyams_utils.interfaces.tales import ITALESExtension
from zope.intid.interfaces import IIntIds
from zope.traversing.interfaces import ITraversable

# import packages
from persistent import Persistent
from pyams_file.property import FileProperty
from pyams_utils.adapter import adapter_config, ContextRequestAdapter, get_annotation_adapter
from pyams_utils.registry import get_utility
from pyams_utils.request import check_request, query_request
from pyramid.security import Allow, ALL_PERMISSIONS, Everyone
from pyramid.threadlocal import get_current_request
from zope.container.contained import Contained
from zope.interface import implementer, Interface
from zope.location import locate


@implementer(IPublicProfile)
class PublicProfile(Persistent, Contained):
    """Public profile persistent class"""

    avatar = FileProperty(IPublicProfile['avatar'])

    def __acl__(self):
        result = [(Allow, ADMIN_USER_ID, ALL_PERMISSIONS)]
        request = query_request()
        if request is not None:
            result.append((Allow, request.principal.id, ALL_PERMISSIONS))
        result.append((Allow, Everyone, PUBLIC_PERMISSION))
        return result


@adapter_config(context=Interface, provides=IPublicProfile)
def public_profile_factory(context):
    request = check_request()
    return IPublicProfile(request.principal)


@adapter_config(context=IPrincipalInfo, provides=IPublicProfile)
def principal_public_profile_factory(principal):
    """Principal public profile factory adapter"""

    def public_profile_callback(profile):
        request = get_current_request()
        if request is not None:
            root = request.root
            intids = get_utility(IIntIds)
            locate(profile, root)  # avoid NotYet exception
            locate(profile, root, '++profile++{0}'.format(intids.register(profile)))

    return get_annotation_adapter(principal, PUBLIC_PROFILE_KEY, PublicProfile,
                                  locate=False, callback=public_profile_callback)


@adapter_config(name='profile', context=(Interface, Interface), provides=ITraversable)
class ProfileTraverser(ContextRequestAdapter):
    """++profile++ namespace traverser"""

    def traverse(self, name, furtherpath=None):
        if not name:
            return IPublicProfile(self.request.principal)
        else:
            intids = get_utility(IIntIds)
            profile = intids.queryObject(int(name))
            return IPublicProfile(profile, None)


@adapter_config(name='public_profile', context=(Interface, Interface), provides=ITALESExtension)
class PublicProfileExtension(ContextRequestAdapter):
    """public_profile TALES extension"""

    def render(self, request=None):
        if request is None:
            request = self.request
        return IPublicProfile(request)
